@@ -21,31 +21,27 @@
 #include <Qt3DExtras/QPhongMaterial>
 #include <Qt3DExtras/QConeMesh>
 #include <Qt3DRender/qcameralens.h>
-#include <Qt3DRender/QCameraSelector>
-#include <Qt3DRender/QClearBuffers>
-#include <Qt3DRender/QLayer>
-#include <Qt3DRender/QLayerFilter>
 #include <Qt3DRender/QPointLight>
 #include <Qt3DRender/QSortPolicy>
 #include <QWidget>
 #include <QScreen>
-#include <QShortcut>
 #include <QFontDatabase>
 #include <ctime>
 #include <QApplication>
 #include <QActionGroup>
 
-#include "qgsmapsettings.h"
+#include "qgs3dmapsettings.h"
 #include "qgs3dmapscene.h"
 #include "qgsterrainentity.h"
 #include "qgscoordinatereferencesystemutils.h"
-#include "qgscoordinatereferencesystem.h"
 #include "qgswindow3dengine.h"
 #include "qgsraycastingutils_p.h"
 #include "qgs3dwiredmesh_p.h"
+#include "qgsframegraph.h"
 #include "qgsabstractterrainsettings.h"
 
-Qgs3DAxis::Qgs3DAxis( Qgs3DMapCanvas *canvas, Qt3DCore::QEntity *parent3DScene, Qgs3DMapScene *mapScene, QgsCameraController *cameraCtrl, Qgs3DMapSettings *map )
+Qgs3DAxis::Qgs3DAxis( Qgs3DMapCanvas *canvas, Qt3DCore::QEntity *parent3DScene, Qgs3DMapScene *mapScene, //
+                      QgsCameraController *cameraCtrl, Qgs3DMapSettings *map )
   : QObject( canvas )
   , mMapSettings( map )
   , mCanvas( canvas )
@@ -53,21 +49,26 @@ Qgs3DAxis::Qgs3DAxis( Qgs3DMapCanvas *canvas, Qt3DCore::QEntity *parent3DScene,
   , mCameraController( cameraCtrl )
   , mCrs( map->crs() )
 {
-  mViewport = constructAxisScene( parent3DScene );
-  mViewport->setParent( mCanvas->activeFrameGraph() );
+  mMapScene->engine()->frameGraph()->registerRenderView( std::make_unique<Qgs3DAxisRenderView>(      //
+                                                           QgsFrameGraph::AXIS3D_RENDERVIEW,         //
+                                                           mCanvas, mCameraController, mMapSettings, //
+                                                           this
+                                                         ),
+                                                         QgsFrameGraph::AXIS3D_RENDERVIEW );
 
+  mRenderView = dynamic_cast<Qgs3DAxisRenderView *>( mMapScene->engine()->frameGraph()->renderView( QgsFrameGraph::AXIS3D_RENDERVIEW ) );
+  Q_ASSERT( mRenderView );
+  constructAxisScene( parent3DScene );
   constructLabelsScene( parent3DScene );
 
+  mTwoDLabelSceneEntity->addComponent( mRenderView->labelLayer() );
+
   connect( cameraCtrl, &QgsCameraController::cameraChanged, this, &Qgs3DAxis::onCameraUpdate );
-  connect( mCanvas, &Qgs3DMapCanvas::widthChanged, this, &Qgs3DAxis::onAxisViewportSizeUpdate );
-  connect( mCanvas, &Qgs3DMapCanvas::heightChanged, this, &Qgs3DAxis::onAxisViewportSizeUpdate );
 
   createAxisScene();
   onAxisViewportSizeUpdate();
 
   init3DObjectPicking();
-
-  createKeyboardShortCut();
 }
 
 Qgs3DAxis::~Qgs3DAxis()
@@ -95,6 +96,8 @@ Qgs3DAxis::~Qgs3DAxis()
       mCubeRoot = nullptr;
       break;
   }
+
+  // render view unregistration will be done by framegraph destructor!
 }
 
 void Qgs3DAxis::init3DObjectPicking()
@@ -106,23 +109,18 @@ void Qgs3DAxis::init3DObjectPicking()
   //   2- connect screencaster results to onTouchedByRay
   //   3- screencaster will be triggered by EventFilter
   mScreenRayCaster = new Qt3DRender::QScreenRayCaster( mAxisSceneEntity );
-  mScreenRayCaster->addLayer( mAxisObjectLayer ); // to only filter on axis objects
+  mScreenRayCaster->addLayer( mRenderView->objectLayer() ); // to only filter on axis objects
   mScreenRayCaster->setFilterMode( Qt3DRender::QScreenRayCaster::AcceptAllMatchingLayers );
   mScreenRayCaster->setRunMode( Qt3DRender::QAbstractRayCaster::SingleShot );
 
   mAxisSceneEntity->addComponent( mScreenRayCaster );
 
   QObject::connect( mScreenRayCaster, &Qt3DRender::QScreenRayCaster::hitsChanged, this, &Qgs3DAxis::onTouchedByRay );
-
-  // we need event filter (see Qgs3DAxis::eventFilter) to handle the mouse click event as this event is not catchable via the Qt3DRender::QObjectPicker
-  mCanvas->installEventFilter( this );
 }
 
-bool Qgs3DAxis::eventFilter( QObject *watched, QEvent *event )
+// will be called by Qgs3DMapCanvas::eventFilter
+bool Qgs3DAxis::handleEvent( QEvent *event )
 {
-  if ( watched != mCanvas )
-    return false;
-
   if ( event->type() == QEvent::MouseButtonPress )
   {
     // register mouse click to detect dragging
@@ -158,11 +156,11 @@ bool Qgs3DAxis::eventFilter( QObject *watched, QEvent *event )
       if ( 2 <= QgsLogger::debugLevel() && event->type() == QEvent::MouseButtonRelease )
       {
         std::ostringstream os;
-        os << "QGS3DAxis: normalized pos: " << normalizedPos << " / viewport: " << mViewport->normalizedRect();
+        os << "QGS3DAxis: normalized pos: " << normalizedPos << " / viewport: " << mRenderView->viewport()->normalizedRect();
         QgsDebugMsgLevel( os.str().c_str(), 2 );
       }
 
-      if ( mViewport->normalizedRect().contains( normalizedPos ) )
+      if ( mRenderView->viewport()->normalizedRect().contains( normalizedPos ) )
       {
         mLastClickedButton = mouseEvent->button();
         mLastClickedPos = mouseEvent->pos();
@@ -261,32 +259,32 @@ void Qgs3DAxis::onTouchedByRay( const Qt3DRender::QAbstractRayCaster::Hits &hits
         {
           case 0: // "East face";
             QgsDebugMsgLevel( "Qgs3DAxis: East face clicked", 2 );
-            onCameraViewChangeEast();
+            mCameraController->rotateCameraToEast();
             break;
 
           case 1: // "West face ";
             QgsDebugMsgLevel( "Qgs3DAxis: West face clicked", 2 );
-            onCameraViewChangeWest();
+            mCameraController->rotateCameraToWest();
             break;
 
           case 2: // "North face ";
             QgsDebugMsgLevel( "Qgs3DAxis: North face clicked", 2 );
-            onCameraViewChangeNorth();
+            mCameraController->rotateCameraToNorth();
             break;
 
           case 3: // "South face";
             QgsDebugMsgLevel( "Qgs3DAxis: South face clicked", 2 );
-            onCameraViewChangeSouth();
+            mCameraController->rotateCameraToSouth();
             break;
 
           case 4: // "Top face ";
             QgsDebugMsgLevel( "Qgs3DAxis: Top face clicked", 2 );
-            onCameraViewChangeTop();
+            mCameraController->rotateCameraToTop();
             break;
 
           case 5: // "Bottom face ";
             QgsDebugMsgLevel( "Qgs3DAxis: Bottom face clicked", 2 );
-            onCameraViewChangeBottom();
+            mCameraController->rotateCameraToBottom();
             break;
 
           default:
@@ -297,49 +295,16 @@ void Qgs3DAxis::onTouchedByRay( const Qt3DRender::QAbstractRayCaster::Hits &hits
   }
 }
 
-Qt3DRender::QViewport *Qgs3DAxis::constructAxisScene( Qt3DCore::QEntity *parent3DScene )
+void Qgs3DAxis::constructAxisScene( Qt3DCore::QEntity *parent3DScene )
 {
-  Qt3DRender::QViewport *axisViewport = new Qt3DRender::QViewport;
-  // parent will be set later
-  // size will be set later
-
   mAxisSceneEntity = new Qt3DCore::QEntity;
   mAxisSceneEntity->setParent( parent3DScene );
   mAxisSceneEntity->setObjectName( "3DAxis_SceneEntity" );
 
-  mAxisObjectLayer = new Qt3DRender::QLayer;
-  mAxisObjectLayer->setObjectName( "3DAxis_ObjectLayer" );
-  mAxisObjectLayer->setParent( mAxisSceneEntity );
-  mAxisObjectLayer->setRecursive( true );
-
-  mAxisCamera = new Qt3DRender::QCamera;
-  mAxisCamera->setParent( mAxisSceneEntity );
-  mAxisCamera->setProjectionType( mCameraController->camera()->projectionType() );
-  mAxisCamera->lens()->setFieldOfView( mCameraController->camera()->lens()->fieldOfView() * 0.5f );
-
+  mAxisCamera = mRenderView->objectCamera();
   mAxisCamera->setUpVector( QVector3D( 0.0f, 1.0f, 0.0f ) );
   mAxisCamera->setViewCenter( QVector3D( 0.0f, 0.0f, 0.0f ) );
   // position will be set later
-
-  Qt3DRender::QLayerFilter *axisLayerFilter = new Qt3DRender::QLayerFilter( axisViewport );
-  axisLayerFilter->addLayer( mAxisObjectLayer );
-
-  Qt3DRender::QCameraSelector *axisCameraSelector = new Qt3DRender::QCameraSelector;
-  axisCameraSelector->setParent( axisLayerFilter );
-  axisCameraSelector->setCamera( mAxisCamera );
-
-  // This ensures to have the labels (Text2DEntity) rendered after the other objects and therefore
-  // avoid any transparency issue on the labels.
-  Qt3DRender::QSortPolicy *sortPolicy = new Qt3DRender::QSortPolicy( axisCameraSelector );
-  QVector<Qt3DRender::QSortPolicy::SortType> sortTypes = QVector<Qt3DRender::QSortPolicy::SortType>();
-  sortTypes << Qt3DRender::QSortPolicy::BackToFront;
-  sortPolicy->setSortTypes( sortTypes );
-
-  Qt3DRender::QClearBuffers *clearBuffers = new Qt3DRender::QClearBuffers( sortPolicy );
-  clearBuffers->setBuffers( Qt3DRender::QClearBuffers::DepthBuffer );
-
-  // cppcheck-suppress memleak
-  return axisViewport;
 }
 
 void Qgs3DAxis::constructLabelsScene( Qt3DCore::QEntity *parent3DScene )
@@ -348,39 +313,10 @@ void Qgs3DAxis::constructLabelsScene( Qt3DCore::QEntity *parent3DScene )
   mTwoDLabelSceneEntity->setParent( parent3DScene );
   mTwoDLabelSceneEntity->setEnabled( true );
 
-  mTwoDLabelCamera = new Qt3DRender::QCamera;
-  mTwoDLabelCamera->setParent( mTwoDLabelSceneEntity );
-  mTwoDLabelCamera->setProjectionType( Qt3DRender::QCameraLens::ProjectionType::OrthographicProjection );
-  // the camera lens parameters are defined by onAxisViewportSizeUpdate()
-
+  mTwoDLabelCamera = mRenderView->labelCamera();
   mTwoDLabelCamera->setUpVector( QVector3D( 0.0f, 0.0f, 1.0f ) );
   mTwoDLabelCamera->setViewCenter( QVector3D( 0.0f, 0.0f, 0.0f ) );
-
   mTwoDLabelCamera->setPosition( QVector3D( 0.0f, 0.0f, 100.0f ) );
-
-  Qt3DRender::QLayer *twoDLayer = new Qt3DRender::QLayer;
-  twoDLayer->setObjectName( "3DAxis_LabelsLayer" );
-  twoDLayer->setRecursive( true );
-  mTwoDLabelSceneEntity->addComponent( twoDLayer );
-
-  Qt3DRender::QLayerFilter *twoDLayerFilter = new Qt3DRender::QLayerFilter;
-  twoDLayerFilter->addLayer( twoDLayer );
-
-  Qt3DRender::QCameraSelector *twoDCameraSelector = new Qt3DRender::QCameraSelector;
-  twoDCameraSelector->setParent( twoDLayerFilter );
-  twoDCameraSelector->setCamera( mTwoDLabelCamera );
-
-  // this ensures to have the labels (Text2DEntity) rendered after the other objects and therefore
-  // avoid any transparency issue on the labels.
-  Qt3DRender::QSortPolicy *sortPolicy = new Qt3DRender::QSortPolicy( twoDCameraSelector );
-  QVector<Qt3DRender::QSortPolicy::SortType> sortTypes = QVector<Qt3DRender::QSortPolicy::SortType>();
-  sortTypes << Qt3DRender::QSortPolicy::BackToFront;
-  sortPolicy->setSortTypes( sortTypes );
-
-  Qt3DRender::QClearBuffers *clearBuffers = new Qt3DRender::QClearBuffers( sortPolicy );
-  clearBuffers->setBuffers( Qt3DRender::QClearBuffers::DepthBuffer );
-
-  twoDLayerFilter->setParent( mViewport );
 }
 
 QVector3D Qgs3DAxis::from3DTo2DLabelPosition( const QVector3D &sourcePos, Qt3DRender::QCamera *sourceCamera, Qt3DRender::QCamera *destCamera )
@@ -431,7 +367,7 @@ void Qgs3DAxis::createAxisScene()
     mAxisRoot = new Qt3DCore::QEntity;
     mAxisRoot->setParent( mAxisSceneEntity );
     mAxisRoot->setObjectName( "3DAxis_AxisRoot" );
-    mAxisRoot->addComponent( mAxisObjectLayer ); // raycaster will filter object containing this layer
+    mAxisRoot->addComponent( mRenderView->objectLayer() ); // raycaster will filter object containing this layer
 
     createAxis( Qt::Axis::XAxis );
     createAxis( Qt::Axis::YAxis );
@@ -440,7 +376,7 @@ void Qgs3DAxis::createAxisScene()
     mCubeRoot = new Qt3DCore::QEntity;
     mCubeRoot->setParent( mAxisSceneEntity );
     mCubeRoot->setObjectName( "3DAxis_CubeRoot" );
-    mCubeRoot->addComponent( mAxisObjectLayer ); // raycaster will filter object containing this layer
+    mCubeRoot->addComponent( mRenderView->objectLayer() ); // raycaster will filter object containing this layer
 
     createCube();
   }
@@ -452,9 +388,11 @@ void Qgs3DAxis::createAxisScene()
     mAxisSceneEntity->setEnabled( false );
     setEnableAxis( false );
     setEnableCube( false );
+    mRenderView->setEnabled( false );
   }
   else
   {
+    mRenderView->setEnabled( true );
     mAxisSceneEntity->setEnabled( true );
     if ( mode == Qgs3DAxisSettings::Mode::Crs )
     {
@@ -496,39 +434,6 @@ void Qgs3DAxis::createAxisScene()
   }
 }
 
-void Qgs3DAxis::createKeyboardShortCut()
-{
-  QgsWindow3DEngine *eng = dynamic_cast<QgsWindow3DEngine *>( mMapScene->engine() );
-  if ( eng )
-  {
-    QWidget *mapCanvas = dynamic_cast<QWidget *>( eng->parent() );
-    if ( !mapCanvas )
-    {
-      QgsLogger::warning( "Qgs3DAxis: no canvas defined!" );
-    }
-    else
-    {
-      QShortcut *shortcutHome = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_1 ), mapCanvas );
-      connect( shortcutHome, &QShortcut::activated, this, [this]() { onCameraViewChangeHome(); } );
-
-      QShortcut *shortcutTop = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_5 ), mapCanvas );
-      connect( shortcutTop, &QShortcut::activated, this, [this]() { onCameraViewChangeTop(); } );
-
-      QShortcut *shortcutNorth = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_8 ), mapCanvas );
-      connect( shortcutNorth, &QShortcut::activated, this, [this]() { onCameraViewChangeNorth(); } );
-
-      QShortcut *shortcutEast = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_6 ), mapCanvas );
-      connect( shortcutEast, &QShortcut::activated, this, [this]() { onCameraViewChangeEast(); } );
-
-      QShortcut *shortcutSouth = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_2 ), mapCanvas );
-      connect( shortcutSouth, &QShortcut::activated, this, [this]() { onCameraViewChangeSouth(); } );
-
-      QShortcut *shortcutWest = new QShortcut( QKeySequence( Qt::CTRL + Qt::Key_4 ), mapCanvas );
-      connect( shortcutWest, &QShortcut::activated, this, [this]() { onCameraViewChangeWest(); } );
-    }
-  }
-}
-
 void Qgs3DAxis::createMenu()
 {
   mMenu = new QMenu();
@@ -601,9 +506,9 @@ void Qgs3DAxis::createMenu()
   hPosGroup->addAction( hPosMiddleAct );
   hPosGroup->addAction( hPosRightAct );
 
-  connect( hPosLeftAct, &QAction::triggered, this, [this]( bool ) { onAxisHorizPositionChanged( Qt::AnchorPoint::AnchorLeft ); } );
-  connect( hPosMiddleAct, &QAction::triggered, this, [this]( bool ) { onAxisHorizPositionChanged( Qt::AnchorPoint::AnchorHorizontalCenter ); } );
-  connect( hPosRightAct, &QAction::triggered, this, [this]( bool ) { onAxisHorizPositionChanged( Qt::AnchorPoint::AnchorRight ); } );
+  connect( hPosLeftAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onHorizontalPositionChanged( Qt::AnchorPoint::AnchorLeft ); } );
+  connect( hPosMiddleAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onHorizontalPositionChanged( Qt::AnchorPoint::AnchorHorizontalCenter ); } );
+  connect( hPosRightAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onHorizontalPositionChanged( Qt::AnchorPoint::AnchorRight ); } );
 
   QMenu *horizPosMenu = new QMenu( QStringLiteral( "Horizontal Position" ), mMenu );
   horizPosMenu->addAction( hPosLeftAct );
@@ -638,9 +543,9 @@ void Qgs3DAxis::createMenu()
   vPosGroup->addAction( vPosMiddleAct );
   vPosGroup->addAction( vPosBottomAct );
 
-  connect( vPosTopAct, &QAction::triggered, this, [this]( bool ) { onAxisVertPositionChanged( Qt::AnchorPoint::AnchorTop ); } );
-  connect( vPosMiddleAct, &QAction::triggered, this, [this]( bool ) { onAxisVertPositionChanged( Qt::AnchorPoint::AnchorVerticalCenter ); } );
-  connect( vPosBottomAct, &QAction::triggered, this, [this]( bool ) { onAxisVertPositionChanged( Qt::AnchorPoint::AnchorBottom ); } );
+  connect( vPosTopAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onVerticalPositionChanged( Qt::AnchorPoint::AnchorTop ); } );
+  connect( vPosMiddleAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onVerticalPositionChanged( Qt::AnchorPoint::AnchorVerticalCenter ); } );
+  connect( vPosBottomAct, &QAction::triggered, this, [this]( bool ) { mRenderView->onVerticalPositionChanged( Qt::AnchorPoint::AnchorBottom ); } );
 
   QMenu *vertPosMenu = new QMenu( QStringLiteral( "Vertical Position" ), mMenu );
   vertPosMenu->addAction( vPosTopAct );
@@ -649,21 +554,22 @@ void Qgs3DAxis::createMenu()
   mMenu->addMenu( vertPosMenu );
 
   // axis view menu
-  QAction *viewHomeAct = new QAction( tr( "&Home" ) + "\t Ctrl+1", mMenu );
-  QAction *viewTopAct = new QAction( tr( "&Top" ) + "\t Ctrl+5", mMenu );
+  // Make sure to sync the key combinations with QgsCameraController::keyboardEventFilter()!
+  QAction *viewHomeAct = new QAction( tr( "&Home" ) + "\t Ctrl+5", mMenu );
+  QAction *viewTopAct = new QAction( tr( "&Top" ) + "\t Ctrl+9", mMenu );
   QAction *viewNorthAct = new QAction( tr( "&North" ) + "\t Ctrl+8", mMenu );
   QAction *viewEastAct = new QAction( tr( "&East" ) + "\t Ctrl+6", mMenu );
   QAction *viewSouthAct = new QAction( tr( "&South" ) + "\t Ctrl+2", mMenu );
   QAction *viewWestAct = new QAction( tr( "&West" ) + "\t Ctrl+4", mMenu );
-  QAction *viewBottomAct = new QAction( tr( "&Bottom" ), mMenu );
+  QAction *viewBottomAct = new QAction( tr( "&Bottom" ) + "\t Ctrl+3", mMenu );
 
-  connect( viewHomeAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeHome );
-  connect( viewTopAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeTop );
-  connect( viewNorthAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeNorth );
-  connect( viewEastAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeEast );
-  connect( viewSouthAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeSouth );
-  connect( viewWestAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeWest );
-  connect( viewBottomAct, &QAction::triggered, this, &Qgs3DAxis::onCameraViewChangeBottom );
+  connect( viewHomeAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToHome );
+  connect( viewTopAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToTop );
+  connect( viewNorthAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToNorth );
+  connect( viewEastAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToEast );
+  connect( viewSouthAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToSouth );
+  connect( viewWestAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToWest );
+  connect( viewBottomAct, &QAction::triggered, mCameraController, &QgsCameraController::rotateCameraToBottom );
 
   QMenu *viewMenu = new QMenu( QStringLiteral( "Camera View" ), mMenu );
   viewMenu->addAction( viewHomeAct );
@@ -702,46 +608,6 @@ void Qgs3DAxis::onAxisModeChanged( Qgs3DAxisSettings::Mode mode )
   mMapSettings->set3DAxisSettings( s );
 }
 
-void Qgs3DAxis::onAxisHorizPositionChanged( Qt::AnchorPoint pos )
-{
-  Qgs3DAxisSettings s = mMapSettings->get3DAxisSettings();
-  s.setHorizontalPosition( pos );
-  mMapSettings->set3DAxisSettings( s );
-}
-
-void Qgs3DAxis::onAxisVertPositionChanged( Qt::AnchorPoint pos )
-{
-  Qgs3DAxisSettings s = mMapSettings->get3DAxisSettings();
-  s.setVerticalPosition( pos );
-  mMapSettings->set3DAxisSettings( s );
-}
-
-void Qgs3DAxis::onCameraViewChange( float pitch, float yaw )
-{
-  QgsVector3D pos = mCameraController->lookingAtPoint();
-  double elevation = 0.0;
-  if ( mMapSettings->terrainRenderingEnabled() )
-  {
-    QgsDebugMsgLevel( "Checking elevation from terrain...", 2 );
-    QVector3D camPos = mCameraController->camera()->position();
-    QgsRayCastingUtils::Ray3D ray( camPos, pos.toVector3D() - camPos, mCameraController->camera()->farPlane() );
-    const QVector<QgsRayCastingUtils::RayHit> hits = mMapScene->terrainEntity()->rayIntersection( ray, QgsRayCastingUtils::RayCastContext() );
-    if ( !hits.isEmpty() )
-    {
-      elevation = hits.at( 0 ).pos.z();
-      QgsDebugMsgLevel( QString( "Computed elevation from terrain: %1" ).arg( elevation ), 2 );
-    }
-    else
-    {
-      QgsDebugMsgLevel( "Unable to obtain elevation from terrain", 2 );
-    }
-  }
-  pos.set( pos.x(), pos.y(), elevation + mMapSettings->terrainSettings()->elevationOffset() );
-
-  mCameraController->setLookingAtPoint( pos, ( mCameraController->camera()->position() - pos.toVector3D() ).length(), pitch, yaw );
-}
-
-
 void Qgs3DAxis::createCube()
 {
   QVector3D minPos = QVector3D( -mCylinderLength * 0.5f, -mCylinderLength * 0.5f, -mCylinderLength * 0.5f );
@@ -990,100 +856,42 @@ void Qgs3DAxis::onAxisSettingsChanged()
   onAxisViewportSizeUpdate();
 }
 
-void Qgs3DAxis::onAxisViewportSizeUpdate( int )
+void Qgs3DAxis::onAxisViewportSizeUpdate()
 {
-  Qgs3DAxisSettings settings = mMapSettings->get3DAxisSettings();
+  mRenderView->onViewportSizeUpdate(); // will call onViewportScaleFactorChanged as callback
 
-  double windowWidth = ( double ) mCanvas->width();
-  double windowHeight = ( double ) mCanvas->height();
-
-  QgsMapSettings set;
-  if ( 2 <= QgsLogger::debugLevel() )
+  // mRenderView->onViewportSizeUpdate() has updated `mTwoDLabelCamera` lens parameters.
+  // The position of the labels needs to be updated.
+  const Qgs3DAxisSettings axisSettings = mMapSettings->get3DAxisSettings();
+  if ( axisSettings.mode() == Qgs3DAxisSettings::Mode::Crs && mAxisRoot->isEnabled() )
   {
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate window w/h: %1px / %2px" ).arg( windowWidth ).arg( windowHeight ), 2 );
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate window physicalDpi %1 (%2, %3)" ).arg( mCanvas->screen()->physicalDotsPerInch() ).arg( mCanvas->screen()->physicalDotsPerInchX() ).arg( mCanvas->screen()->physicalDotsPerInchY() ), 2 );
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate window logicalDotsPerInch %1 (%2, %3)" ).arg( mCanvas->screen()->logicalDotsPerInch() ).arg( mCanvas->screen()->logicalDotsPerInchX() ).arg( mCanvas->screen()->logicalDotsPerInchY() ), 2 );
+    updateAxisLabelPosition();
+  }
+}
 
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate window pixel ratio %1" ).arg( mCanvas->screen()->devicePixelRatio() ), 2 );
-
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate set pixel ratio %1" ).arg( set.devicePixelRatio() ), 2 );
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate set outputDpi %1" ).arg( set.outputDpi() ), 2 );
-    QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate set dpiTarget %1" ).arg( set.dpiTarget() ), 2 );
+void Qgs3DAxis::onViewportScaleFactorChanged( double scaleFactor )
+{
+  // if the axis scene has not been created, don't do anything
+  if ( !mAxisRoot || !mCubeRoot )
+  {
+    return;
   }
 
-  // default viewport size in pixel according to 92 dpi
-  double defaultViewportPixelSize = ( ( double ) settings.defaultViewportSize() / 25.4 ) * 92.0;
-
-  // computes the viewport size according to screen dpi but as the viewport size growths too fast
-  // then we limit the growth by using a factor on the dpi difference.
-  double viewportPixelSize = defaultViewportPixelSize + ( ( double ) settings.defaultViewportSize() / 25.4 ) * ( mCanvas->screen()->physicalDotsPerInch() - 92.0 ) * 0.7;
-  QgsDebugMsgLevel( QString( "onAxisViewportSizeUpdate viewportPixelSize %1" ).arg( viewportPixelSize ), 2 );
-  double widthRatio = viewportPixelSize / windowWidth;
-  double heightRatio = widthRatio * windowWidth / windowHeight;
-
-  QgsDebugMsgLevel( QString( "3DAxis viewport ratios width: %1% / height: %2%" ).arg( widthRatio ).arg( heightRatio ), 2 );
-
-  if ( heightRatio * windowHeight < viewportPixelSize )
+  if ( scaleFactor > 0.0 )
   {
-    heightRatio = viewportPixelSize / windowHeight;
-    widthRatio = heightRatio * windowHeight / windowWidth;
-    QgsDebugMsgLevel( QString( "3DAxis viewport, height too small, ratios adjusted to width: %1% / height: %2%" ).arg( widthRatio ).arg( heightRatio ), 2 );
-  }
+    Qgs3DAxisSettings settings = mMapSettings->get3DAxisSettings();
+    if ( settings.mode() == Qgs3DAxisSettings::Mode::Crs )
+      setEnableAxis( true );
+    else if ( settings.mode() == Qgs3DAxisSettings::Mode::Cube )
+      setEnableCube( true );
 
-  if ( heightRatio > settings.maxViewportRatio() || widthRatio > settings.maxViewportRatio() )
-  {
-    QgsDebugMsgLevel( "viewport takes too much place into the 3d view, disabling it", 2 );
-    // take too much place into the 3d view
-    mViewport->setEnabled( false );
-    setEnableCube( false );
-    setEnableAxis( false );
+    mAxisScaleFactor = scaleFactor;
+    QgsDebugMsgLevel( QString( "3DAxis viewport mAxisScaleFactor %1" ).arg( mAxisScaleFactor ), 2 );
   }
   else
   {
-    // will be used to adjust the axis label translations/sizes
-    mAxisScaleFactor = viewportPixelSize / defaultViewportPixelSize;
-    QgsDebugMsgLevel( QString( "3DAxis viewport mAxisScaleFactor %1" ).arg( mAxisScaleFactor ), 2 );
-
-    if ( !mViewport->isEnabled() )
-    {
-      if ( settings.mode() == Qgs3DAxisSettings::Mode::Crs )
-        setEnableAxis( true );
-      else if ( settings.mode() == Qgs3DAxisSettings::Mode::Cube )
-        setEnableCube( true );
-    }
-    mViewport->setEnabled( true );
-
-    float xRatio = 1.0f;
-    float yRatio = 1.0f;
-    if ( settings.horizontalPosition() == Qt::AnchorPoint::AnchorLeft )
-      xRatio = 0.0f;
-    else if ( settings.horizontalPosition() == Qt::AnchorPoint::AnchorHorizontalCenter )
-      xRatio = 0.5f - static_cast<float>( widthRatio ) / 2.0f;
-    else
-      xRatio = 1.0f - static_cast<float>( widthRatio );
-
-    if ( settings.verticalPosition() == Qt::AnchorPoint::AnchorTop )
-      yRatio = 0.0f;
-    else if ( settings.verticalPosition() == Qt::AnchorPoint::AnchorVerticalCenter )
-      yRatio = 0.5f - static_cast<float>( heightRatio ) / 2.0f;
-    else
-      yRatio = 1.0f - static_cast<float>( heightRatio );
-
-    QgsDebugMsgLevel( QString( "Qgs3DAxis: update viewport: %1 x %2 x %3 x %4" ).arg( xRatio ).arg( yRatio ).arg( widthRatio ).arg( heightRatio ), 2 );
-    mViewport->setNormalizedRect( QRectF( xRatio, yRatio, widthRatio, heightRatio ) );
-
-    if ( settings.mode() == Qgs3DAxisSettings::Mode::Crs )
-    {
-      const float halfWidthSize = static_cast<float>( windowWidth * widthRatio / 2.0 );
-      const float halfHeightSize = static_cast<float>( windowWidth * widthRatio / 2.0 );
-      mTwoDLabelCamera->lens()->setOrthographicProjection(
-        -halfWidthSize, halfWidthSize,
-        -halfHeightSize, halfHeightSize,
-        mTwoDLabelCamera->lens()->nearPlane(), mTwoDLabelCamera->lens()->farPlane()
-      );
-
-      updateAxisLabelPosition();
-    }
+    setEnableCube( false );
+    setEnableAxis( false );
   }
 }
 
